#include "StdAfx.h"
#include "Controls.h"
#include "Resource.h"

const TCHAR szCGButtonClassName[] = TEXT("CGButtonClassName");
const TCHAR szCPButtonClassName[] = TEXT("CPButtonClassName");
const TCHAR szCPanelClassName[] = TEXT("CPanelClassName");
const TCHAR szCGripClassName[] = TEXT("CGripClassName");

/********************************************************************
 *
 * CBaseControl
 *
 ********************************************************************/

void CBaseControl::Invalidate()
{
	::InvalidateRect(Handle, NULL, TRUE);
}
void CBaseControl::Move(int x, int y, int cx, int cy)
{
	//SetWindowPos(Handle, HWND_TOP, x, y, 10, 10, /*SWP_NOZORDER |*/ SWP_NOSIZE | SWP_NOOWNERZORDER | SWP_SHOWWINDOW);
	MoveWindow(Handle, x, y, cx, cy, TRUE);
}

BOOL CBaseControl::Show()
{
	return ShowWindow(Handle, SW_SHOW);
}

BOOL CBaseControl::Hide()
{
	return ShowWindow(Handle, SW_HIDE);
}

BOOL CBaseControl::Enable()
{
	return EnableWindow(Handle, TRUE);
}

BOOL CBaseControl::Disable()
{
	return EnableWindow(Handle, FALSE);
}

/********************************************************************
 *
 * CWinControl
 *
 ********************************************************************/



LRESULT CALLBACK CWinControl::initWindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (message == WM_NCCREATE)
	{
		CWinControl *this_ = (CWinControl*)((LPCREATESTRUCT)lParam)->lpCreateParams;
		if (this_)
		{
#ifdef _WIN64
			::SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)this_);
			::SetWindowLongPtr(hWnd, GWLP_WNDPROC, (LONG_PTR)tWndProc);
#else
			::SetWindowLongPtr(hWnd, GWLP_USERDATA, PtrToLong(this_));
			::SetWindowLongPtr(hWnd, GWLP_WNDPROC, PtrToLong(tWndProc));
#endif
			return this_->WndProc(hWnd, message, wParam, lParam); 

		}
	}
	return ::DefWindowProc(hWnd, message, wParam, lParam); 
}

LRESULT CALLBACK CWinControl::tWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (message == WM_DESTROY || message == WM_NCDESTROY)
		return FALSE;
	//CWinControl* this_ = (CWinControl*)GetWindowLongPtr(hWnd, GWLP_USERDATA);
	CWinControl* this_ = (CWinControl*)LongToPtr(GetWindowLongPtr(hWnd, GWLP_USERDATA));
	if (this_)
		return this_->WndProc(hWnd, message, wParam, lParam); 
	return FALSE;
}

LRESULT CWinControl::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	return DefWindowProc(hWnd, message, wParam, lParam);
}

/********************************************************************
 *
 * CPPanel
 *
 ********************************************************************/

CPPanel::CPPanel()
{
	m_bEnable = FALSE;
}

void CPPanel::SetEnable(BOOL bEnable)
{
	if (m_bEnable != bEnable)
	{
		m_bEnable = bEnable;
		Invalidate();
	}
}

HWND CPPanel::CreatePanel(HWND hWnd, HINSTANCE hInst, int id)
{
	m_hWndOwner = hWnd;
	m_bMouseOver = FALSE;
	m_bLButtonDown = FALSE;
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcex.lpfnWndProc	= initWindowProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInst;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= NULL; //(HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szCPanelClassName;
	wcex.hIconSm		= NULL;
	RegisterClassEx(&wcex);
	Handle = CreateWindow(
		//WS_EX_TOPMOST,
		szCPanelClassName,							//Rg[
		NULL,										//ŏɕ\镶
		WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE,					//EBhEX^C
		0,0,2,2,									//x,y,w,h
		hWnd,										//eEBhẼnh
		(HMENU)IntToPtr(id),									//Rg[ ID
		hInst,										//CX^XEnh
		this);										//ʏ NULLALPCREATESTRUCTQƂ this
	//FontKp
	//SendMessage(Handle, WM_SETFONT, (WPARAM) hf, MAKELPARAM(true, 0));
	return Handle;
}
//int CPPanel::DrawPath(Graphics *g, Pen *pen, LinearGradientBrush *brush, RECT *rc)
int CPPanel::DrawPath(Graphics *g, Pen *pen, LinearGradientBrush *brush, SolidBrush *brush2, RECT *rc)
{
	return 0;
}

void CPPanel::PaintImage(HWND hWnd)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(hWnd, &ps);
	RECT rc;
	GetClientRect(Handle, &rc);
	Rect gprc(0, 0, rc.right, rc.bottom);
	Image *image = new Bitmap(rc.right, rc.bottom, PixelFormat32bppARGB);
	Graphics *tg = new Graphics(image);
	tg->SetSmoothingMode(SmoothingModeAntiAlias); //tg->Clear(Color(0, 255, 255, 255));
	Pen *pen;
	LinearGradientBrush *brush;
	SolidBrush *brush2;
	//brush2 = new HatchBrush(HatchStyleCross, Color::White, Color::Transparent);
	//brush2 = new SolidBrush(Color::White);
	if (!m_bEnable) {
		pen = new Pen(Color(127, 0, 0, 0));
		brush = new LinearGradientBrush(gprc, Color(127, 255, 255, 255), Color(63, 0, 0, 0), LinearGradientModeVertical);
		brush2 = new SolidBrush(Color(127, 255, 255, 255));
	} else if (m_bLButtonDown) {
		pen = new Pen(Color(255, 0, 0, 0));
		brush = new LinearGradientBrush(gprc, Color::White, Color::Black, LinearGradientModeVertical);
		brush2 = new SolidBrush(Color::White);
	} else if ( m_bMouseOver) {
		pen = new Pen(Color(191, 0, 0, 0));
		brush = new LinearGradientBrush(gprc, Color(191, 255, 255, 255), Color(127, 0, 0, 0), LinearGradientModeVertical);
		brush2 = new SolidBrush(Color(191, 255, 255, 255));
	} else {
		pen = new Pen(Color(31, 0, 0, 0));
		brush = new LinearGradientBrush(gprc, Color(127, 255, 255, 255), Color(63, 0, 0, 0), LinearGradientModeVertical);
		brush2 = new SolidBrush(Color(127, 255, 255, 255));
	}
#ifdef _WINXP
	tg->FillRectangle(&SolidBrush(Color::Black), -1, -1, rc.right+1, rc.bottom+1);
#endif
	DrawPath(tg, pen, brush, brush2, &rc); //pĊG
	Graphics g(hdc);
	g.SetCompositingMode(CompositingModeSourceCopy);
	g.SetCompositingQuality(CompositingQualityInvalid);
	g.DrawImage(image, 0, 0, rc.right, rc.bottom);
	delete brush2;
	delete brush;
	delete pen;
	delete tg;
	delete image;
	EndPaint(hWnd, &ps);
}

LRESULT CPPanel::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		case WM_PAINT:			
			PaintImage(hWnd);
            break;
		case WM_LBUTTONUP:
			::ReleaseCapture();
			m_bMouseOver = FALSE;
			m_bLButtonDown = FALSE;
			OnLButtonUp();
			Invalidate();
			break;
		case WM_LBUTTONDOWN:
			SetCapture(Handle);
			m_bLButtonDown = TRUE;
			OnLButtonDown();
			Invalidate();
			break;
		case WM_MOUSEMOVE:
			POINT pt;
			RECT rc;
			::GetClientRect(hWnd, &rc);
			::GetCursorPos(&pt); 
			::ScreenToClient(hWnd, &pt);
			if (PtInRect(&rc, pt))
			{
				if (!m_bMouseOver)
				{
					::SetCapture(Handle);
					m_bMouseOver = TRUE;
					//m_LButtonDown = FALSE;
					Invalidate();
				}
			} else {
				//m_MouseOver = FALSE;
				//m_LButtonDown = FALSE;
				//::ReleaseCapture();
				if (!m_bLButtonDown) {
					m_bMouseOver = FALSE;
					ReleaseCapture();
					Invalidate();
				}
			}
			if (m_bLButtonDown)
				OnMouseMove();
			break;
		case WM_CREATE:
			m_bMouseOver = FALSE;
			break;
		case WM_SIZE:
			OnSize();
			break;
        default:
            break;
    }
	return DefWindowProc(hWnd, message, wParam, lParam);
}

/********************************************************************
 *
 * CGButton
 *
 ********************************************************************/

HWND CGButton::CreateButton(HWND hWnd, HINSTANCE hInst, RECT rc, int id, int id_Cold, int id_Hot, int id_Push)
{
	HotImg  = new Bitmap(hInst, MAKEINTRESOURCE(id_Hot));
	ColdImg = new Bitmap(hInst, MAKEINTRESOURCE(id_Cold));
	PushImg = new Bitmap(hInst, MAKEINTRESOURCE(id_Push));
	//Gdiplus::Status::
	m_Width = rc.right - rc.left;
	m_Height = rc.bottom - rc.top;
	m_hWndOwner = hWnd;
	m_ButtonID = id;
	m_MouseOver = FALSE;
	m_LButtonDown = FALSE;
	//WNDCLASS
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcex.lpfnWndProc	= initWindowProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInst;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= NULL;
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szCGButtonClassName;
	wcex.hIconSm		= NULL;
	RegisterClassEx(&wcex);
	//ECh쐬
	Handle = CreateWindow(
		szCGButtonClassName,
		NULL,
		WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE,
		rc.left, rc.top, rc.right, rc.top,
		hWnd,
		(HMENU)IntToPtr(id),
		hInst,
		this);
	return Handle;
}

void CGButton::PaintImage(HWND hWnd)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(hWnd, &ps);
	Rect gprc(0, 0, m_Width, m_Height);
	ImageAttributes att;// for TransColor
	//att.SetColorKey(Color(255,0,255), Color(255,0,255));
	att.SetColorKey(Color::White, Color::White);
	Graphics g(hdc);
	g.Clear(Color::Black);
	if (m_LButtonDown)
		g.DrawImage(PushImg, gprc, 0, 0, m_Width, m_Height, UnitPixel, &att);
	else if (m_MouseOver)
		g.DrawImage(HotImg, gprc, 0, 0, m_Width, m_Height, UnitPixel, &att);
	else
		g.DrawImage(ColdImg, gprc, 0, 0, m_Width, m_Height, UnitPixel, &att);
	EndPaint(hWnd, &ps);
}

LRESULT CGButton::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	POINT pt;
	RECT rc;
    switch (message) {
        case WM_PAINT:			
			PaintImage(hWnd);
            break;
		case WM_LBUTTONUP:
			::ReleaseCapture();
			m_MouseOver = FALSE;
			m_LButtonDown = FALSE;
			Invalidate();
			//OnClick();
			SendMessage(m_hWndOwner, WM_BUTTONNOTIFY, m_ButtonID, 0L);
			break;
		case WM_LBUTTONDOWN:
			m_LButtonDown = TRUE;
			Invalidate();
			break;
		case WM_MOUSEMOVE:
			::GetClientRect(hWnd, &rc);
			::GetCursorPos(&pt); 
			::ScreenToClient(hWnd, &pt);
			if (PtInRect(&rc, pt))
			{
				if (!m_MouseOver)
				{
					::SetCapture(Handle);
					m_MouseOver = TRUE;
					//m_LButtonDown = FALSE;
					Invalidate();
				}
			} else {
				m_MouseOver = FALSE;
				m_LButtonDown = FALSE;
				::ReleaseCapture();
				Invalidate();
			}
			break;
		case WM_CREATE:
			m_MouseOver = FALSE;
			break;
        default:
            break;
    }
	return DefWindowProc(hWnd, message, wParam, lParam);
}

CGButton::CGButton()
{
}

CGButton::~CGButton()
{
	DeleteGlyph();
}

BOOL CGButton::DeleteGlyph()
{
	if (HotImg)
		delete HotImg;
	if (ColdImg)
		delete ColdImg;
	if (PushImg)
		delete PushImg;
	return TRUE;
}


void CGButton::SetGlyph(HINSTANCE hInst, int id_Cold, int id_Hot, int id_Push)
{
	DeleteGlyph();
	HotImg  = new Bitmap(hInst, MAKEINTRESOURCE(id_Hot));
	ColdImg = new Bitmap(hInst, MAKEINTRESOURCE(id_Cold));
	PushImg = new Bitmap(hInst, MAKEINTRESOURCE(id_Push));
	Invalidate();
}

/********************************************************************
 *
 * CPButton
 *
 ********************************************************************/

HWND CPButton::CreateButton(HWND hWnd, HINSTANCE hInst, RECT rc, int id)
{
	m_Width = rc.right - rc.left;
	m_Height = rc.bottom - rc.top;
	m_hWndOwner = hWnd;
	m_ButtonID = id;
	m_MouseOver = FALSE;
	m_LButtonDown = FALSE;
	m_bDown = FALSE;
	//WNDCLASS
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcex.lpfnWndProc	= initWindowProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInst;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= NULL;
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szCPButtonClassName;
	wcex.hIconSm		= NULL;
	RegisterClassEx(&wcex);
	//ECh쐬
	Handle = CreateWindow(
		szCPButtonClassName,
		NULL,
		WS_CHILD /*| WS_CLIPSIBLINGS*/ | WS_VISIBLE,
		rc.left, rc.top, rc.right, rc.top,
		hWnd,
		(HMENU)IntToPtr(id),
		hInst,
		this);
	return Handle;
}

int CPButton::DrawPath(Gdiplus::Graphics *g, Gdiplus::Pen *pen, Gdiplus::LinearGradientBrush *brush, Rect rc)
{
	return 0;
}

void CPButton::PaintImage(HWND hWnd)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(hWnd, &ps);
	Rect gprc(0, 0, m_Width, m_Height);
	Image *image = new Bitmap(m_Width, m_Height, PixelFormat32bppARGB);
	Graphics *tg = new Graphics(image);
	tg->SetSmoothingMode(SmoothingModeAntiAlias);
	Pen *pen;
	LinearGradientBrush *brush;
	if ( m_bDown) {
		if ( m_MouseOver) 
		{
			pen = new Pen(Color(255, 48, 48, 48));
			brush = new LinearGradientBrush(gprc, Color(127, 127, 127, 127), Color(127, 127, 255, 127), LinearGradientModeForwardDiagonal);
		} else {
			pen = new Pen(Color(191, 48, 48, 48));
			brush = new LinearGradientBrush(gprc, Color(127, 255, 255, 255), Color(127, 127, 255, 127), LinearGradientModeForwardDiagonal);
		}
	} else if (m_LButtonDown) {
		pen = new Pen(Color(255, 48, 48, 48));
		brush = new LinearGradientBrush(gprc, Color::White, Color::Black, LinearGradientModeForwardDiagonal);
	} else if ( m_MouseOver) {
		pen = new Pen(Color(255, 48, 48, 48));
		brush = new LinearGradientBrush(gprc, Color(127, 255, 255, 255), Color(127, 0, 0, 0), LinearGradientModeForwardDiagonal);
	} else {
		pen = new Pen(Color(191, 48, 48, 48));
		brush = new LinearGradientBrush(gprc, Color(31, 255, 255, 255), Color(31, 0, 0, 0), LinearGradientModeForwardDiagonal);
	}
#ifdef _WINXP
	tg->FillRectangle(&SolidBrush(Color::Black), -1, -1, m_Width+1, m_Height+1);
#endif
	DrawPath(tg, pen, brush, gprc); //pĊG
	Graphics g(hdc);
	g.SetCompositingMode(CompositingModeSourceCopy);
	g.SetCompositingQuality(CompositingQualityInvalid);
	g.DrawImage(image, 0, 0, m_Width, m_Height);
	delete brush;
	delete pen;
	delete tg;
	delete image;
	EndPaint(hWnd, &ps);
}

LRESULT CPButton::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	POINT pt;
	RECT rc;
    switch (message) {
        case WM_PAINT:			
			PaintImage(hWnd);
            break;
		case WM_LBUTTONUP:
			::ReleaseCapture();
			m_MouseOver = FALSE;
			m_LButtonDown = FALSE;
			Invalidate();
			//OnClick();
			SendMessage(m_hWndOwner, WM_BUTTONNOTIFY, m_ButtonID, 0L);
			break;
		case WM_LBUTTONDOWN:
			m_LButtonDown = TRUE;
			Invalidate();
			break;
		case WM_MOUSEMOVE:
			::GetClientRect(hWnd, &rc);
			::GetCursorPos(&pt); 
			::ScreenToClient(hWnd, &pt);
			if (PtInRect(&rc, pt))
			{
				if (!m_MouseOver)
				{
					::SetCapture(Handle);
					m_MouseOver = TRUE;
					//m_LButtonDown = FALSE;
					Invalidate();
				}
			} else {
				m_MouseOver = FALSE;
				m_LButtonDown = FALSE;
				::ReleaseCapture();
				Invalidate();
			}
			break;
		case WM_CREATE:
			m_MouseOver = FALSE;
			break;
        default:
            break;
    }
	return DefWindowProc(hWnd, message, wParam, lParam);
}

void CPButton::Down(BOOL down)
{
	m_bDown = down;
	Invalidate();
}

BOOL CPButton::isDown()
{
	return m_bDown;
}

/********************************************************************
 *
 * CGrip
 *
 ********************************************************************/

HWND CGrip::CreateGrip(HWND hWnd, HINSTANCE hInst, RECT rc, int id)
{
	m_hWndOwner = hWnd;
	m_MouseOver = FALSE;
	m_LButtonDown = FALSE;
	//WNDCLASS
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcex.lpfnWndProc	= initWindowProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInst;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_CROSS);
	wcex.hbrBackground	= NULL;
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szCGripClassName;
	wcex.hIconSm		= NULL;
	RegisterClassEx(&wcex);
	//ECh쐬
	Handle = CreateWindow(
		szCGripClassName,
		NULL,
		WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE,
		rc.left, rc.top, rc.right, rc.top,
		hWnd,
		(HMENU)IntToPtr(id),
		hInst,
		this);
	return Handle;
}

void CGrip::PaintImage(HWND hWnd)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(hWnd, &ps);
	RECT rc;
	GetClientRect(Handle, &rc);
	Image *image = new Bitmap(rc.right, rc.bottom, PixelFormat32bppARGB);
	Graphics *tg = new Graphics(image);
	//tg->SetSmoothingMode(SmoothingModeAntiAlias);
	Pen *pen;
	if (m_LButtonDown) {
		pen = new Pen(Color(255, 48, 48, 48));
	} else if ( m_MouseOver) {
		pen = new Pen(Color(191, 48, 48, 48));
	} else {
		pen = new Pen(Color(63, 48, 48, 48));
	}
	//RRɐ
	DrawPath(tg, pen, rc);
	Graphics g(hdc);
	g.SetCompositingMode(Gdiplus::CompositingModeSourceCopy);
	g.DrawImage(image, 0, 0, rc.right, rc.bottom);
	delete pen;
	delete tg;
	delete image;
	EndPaint(hWnd, &ps);
}

LRESULT CGrip::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	//POINT pt;
	//RECT rc;
    switch (message) {
        case WM_PAINT:			
			PaintImage(hWnd);
            break;
		case WM_LBUTTONUP:
			//::ReleaseCapture();
			//m_MouseOver = FALSE;
			m_LButtonDown = FALSE;
			Invalidate();
			break;
		case WM_LBUTTONDOWN:
			if (!m_LButtonDown)
			{
				m_LButtonDown = TRUE;
				::GetCursorPos(&m_DownPoint);
				m_DownX = m_DownPoint.x;
				//m_DownX = LOWORD(lParam);
				m_MoveSize = 0;
				Invalidate();
			}
			break;
		case WM_MOUSEMOVE:
			if (m_LButtonDown)
			{
				//m_MoveSize = LOWORD(lParam) - m_DownX;
				::GetCursorPos(&m_DownPoint);
				m_MoveSize = m_DownPoint.x - m_DownX;
				OnMove(m_MoveSize);
			} else {
				POINT pt;
				RECT rc;
				::GetClientRect(hWnd, &rc);
				::GetCursorPos(&pt); 
				::ScreenToClient(hWnd, &pt);
				if (PtInRect(&rc, pt))
				{
					if (!m_MouseOver)
					{
						::SetCapture(Handle);
						m_MouseOver = TRUE;
						Invalidate();
					}
				} else {
					m_MouseOver = FALSE;
					m_LButtonDown = FALSE;
					::ReleaseCapture();
					Invalidate();
				}
			}
			break;
		case WM_CREATE:
			m_MouseOver = FALSE;
			break;
        default:
            break;
    }
	return DefWindowProc(hWnd, message, wParam, lParam);
}
